# FSGS - Common functionality for FS Game System.
# Copyright (C) 2013-2019  Frode Solheim <frode@solheim.dev>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

PLATFORM_AMIGA = "amiga"
PLATFORM_ARCADE = "arcade"
PLATFORM_ATARI = "atari"
PLATFORM_A2600 = "a2600"
PLATFORM_A5200 = "a5200"
PLATFORM_A7800 = "a7800"
PLATFORM_C64 = "c64"
PLATFORM_CD32 = "cd32"
PLATFORM_CDTV = "cdtv"
PLATFORM_CPC = "cpc"
PLATFORM_DOS = "dos"
PLATFORM_GB = "gb"
PLATFORM_GBA = "gba"
PLATFORM_GBC = "gbc"
PLATFORM_GAME_GEAR_ = "sgg"
# PLATFORM_LYNX = "lynx"
PLATFORM_SMS = "sms"
PLATFORM_SMD = "smd"
PLATFORM_N64 = "n64"
PLATFORM_NES = "nes"
PLATFORM_SNES = "snes"
PLATFORM_TG16 = "tg16"
PLATFORM_ZXS = "zxs"
